/* ---------------------------------
This is a demo of several things:
1) drawing a bitmapped 256 color background
2) displaying two sprites
3) using the mt.h Rand() function
4) showing the speed differences in stdlib.h rand() and mt.h Rand()

by Jae <jaemunce@tranquility.net> */

// one include to rule them all...
#include "mygba.h"

// include the graphics...
#include "gfx/bg.raw.c"
#include "gfx/bg.pal.c"
#include "gfx/master.pal.c"
#include "gfx/enemy.raw.c"
#include "gfx/monster.raw.c"

// some options for randomisation
#include "mt.h"
#include "time.h"
#include "stdlib.h"

/* Program Entry Point */

u8 newframe=0;
u32 frames=0;
u16 myX=16;
u16 myY=16;
u16 eX;
u16 eY;
s8 fadeAmt=0;
int fadeDir=0; //fade out, 0, fade in, 1

void vblFunc()
{
	// Copy the objs into OAM once per frame
	ham_CopyObjToOAM();

	ham_RotBg(2,frames%360,120,80,0x100+(SIN_SFP32[180+frames%180]));
	
	/* Fade Support will go here some day...
	if(fadeAmt >= 16) //The number of colors I actually use...
	{
		fadeAmt--;
	}else{
		fadeAmt++;
	}
	ham_FadePal(0,fadeAmt); */

   	//signify new frame	
	newframe=1;
	frames++;
}

void AgbMain(void)
{

	u8 playerSprite;
	u8 enemySprite;

	//on init to rule them all...
	ham_Init();

	//for debugging purposes only...
	//ham_InitText(0);

	//set the BG mode to bitmapped...
	ham_SetBgMode(4); //it's a 256 color bg...

	//dma the BG pic...
	TOOL_DMA1_SET(&bg_Bitmap,MEM_BG_PTR,SIZEOF_32BIT(bg_Bitmap),DMA_TRANSFER_32BIT,DMA_STARTAT_NOW)

	//bring in dat dere bg palette, no?
	ham_LoadBGPal(&bg_Pal,SIZEOF_8BIT(bg_Pal));

	//load up the sprites' shared pallete
	ham_LoadObjPal(&master_Palette,SIZEOF_16BIT(master_Palette));

	//randomly place the enemy...
	seedMT(time(NULL)); //seed the mt.h
	eX = Rand(15)*16;
	eY = Rand(10)*16;
	
	myX = Rand(15)*16;
	myY = Rand(10)*16;	

	//init sprite 0 with all attributes
	playerSprite = ham_CreateObj(&monster_Bitmap[16*16*0],0,1,OBJ_MODE_NORMAL,1,0,0,0,0,myX,myY);
	//init sprite 1
	enemySprite = ham_CreateObj(&enemy_Bitmap[16*16*0],0,1,OBJ_MODE_NORMAL,1,0,0,0,0,eX,eY);

	// start hblFunc every frame
	ham_StartIntHandler(INT_TYPE_VBL,&vblFunc);

	while(1)
    	{
		if(F_CTRLINPUT_A_PRESSED) //use the faster mt.h
		{
			eX = Rand(15)*16;
			eY = Rand(10)*16;
			ham_SetObjX(enemySprite,eX);
			ham_SetObjY(enemySprite,eY);
		}
		if(F_CTRLINPUT_B_PRESSED) //use the MUCH SLOWER stdlib.h
		{
			srand(time(NULL)); //seed it...
			myX = (rand()%15)*16; //this thing sucks...
			myY = (rand()%10)*16;
			ham_SetObjX(playerSprite,myX); //display...
			ham_SetObjY(playerSprite,myY);
		}
		newframe=0; //update the frames so that the rotation occurs properly.
    }    
}